import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;

/** DemoCustomButtonUI3 - similar to <code>DemoCustomButtonUI2</code>
* except using round buttons.
* <p>
*
* Screen snaps:<p>
*
* <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Launch...</code><br>
* <center><img src="DemoCustomButtonUI3-1.gif"></center><p>
* 
* <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;After some input, custom button armed/pressed...</code><br>
* <center><img src="DemoCustomButtonUI3-2.gif"></center><p>
* 
* @see <a href="DemoCustomButtonUI3.java">source code</a>
* @author Scott MacKenzie, 2007-2011
*/
public class DemoCustomButtonUI3
{
   public static void main(String[] args)
   {
      // use look and feel for my system (Win32)
      try {
         UIManager.setLookAndFeel(
            UIManager.getSystemLookAndFeelClassName());
      } catch (Exception e) {}

      DemoCustomButtonUI3Frame frame = new DemoCustomButtonUI3Frame();
      frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      frame.setTitle("DemoCustomButtonUI3");
      frame.pack();

      // position application window in the center of the display
      Dimension d1 = frame.getSize();
      Dimension d2 = Toolkit.getDefaultToolkit().getScreenSize();
      int x = (d2.width - d1.width) / 2;
      int y = (d2.height - d1.height) / 2; 
      frame.setLocation(new Point(x, y));

      frame.setVisible(true);
   }
}

class DemoCustomButtonUI3Frame extends JFrame implements ActionListener
{
    // the following avoids a "warning" with Java 1.5.0 complier (?)
	static final long serialVersionUID = 42L;

   final String[] LABELS = { "1", "2", "3", "4", "5" };
   final int N = LABELS.length;
   JButton[] b;
   JTextArea input;

   public DemoCustomButtonUI3Frame()
   {
      final Color BLUE = new Color(47, 79, 136);

      // ----------------------------------
      // construct and configure components
      // ----------------------------------

      b = new JButton[N];

      for (int i = 0; i < N; ++i)
      {
         b[i] = new JButton(LABELS[i]);
         b[i].setUI(new CustomButtonUI3());
         b[i].addActionListener(this);
      }

      input = new JTextArea(3, 20);

      // -----------------
      // layout components
      // -----------------

      JPanel panel = new JPanel();
      for (int i = 0; i < N; ++i)
      {
         panel.add(b[i]);
      }
      panel.setBorder(BorderFactory.createEmptyBorder(25, 25, 25, 25));
      panel.setBackground(BLUE);

      // make panel this JFrame's content pane

      JPanel p = new JPanel(new BorderLayout());
      p.add(input, "North");
      p.add(panel, "Center");

      this.setContentPane(p);
   }

   // -------------------------------
   // implement ActionListener method
   // -------------------------------

   public void actionPerformed(ActionEvent ae)
   {
      String s = ((JButton)ae.getSource()).getText();
      input.append(s);
   }

   // -----------
   // inner class
   // -----------

   class CustomButtonUI3 extends ButtonUI
   {
      final Insets I = new Insets(10, 10, 10, 10);

      public void installUI(JComponent c)
      {
         AbstractButton b = (AbstractButton)c;
         BasicButtonListener listener = new BasicButtonListener(b);
         b.addMouseListener(listener);
      }

      public void uninstallUI(JComponent c)
      {
         AbstractButton b = (AbstractButton)c;
         BasicButtonListener listener = new BasicButtonListener(b);
         b.removeMouseListener(listener);
      }

      public Insets getDefaultMargin(AbstractButton ab)
      {
         return I;
      }

      public Insets getInsets(JComponent c)
      {
         return I;
      }

      public Dimension getMaximumSize(JComponent c)
      {
         return this.getPreferredSize(c);
      }

      public Dimension getMinimumSize(JComponent c)
      {
         return this.getPreferredSize(c);
      }

      public Dimension getPreferredSize(JComponent c)
      {
         Graphics g = c.getGraphics();
         FontMetrics fm = g.getFontMetrics();

         Dimension d = new Dimension();
         d.width = fm.getAscent() + I.left + I.right;
         d.height = fm.getAscent() + I.top + I.bottom; 
         return d;
      }

      public void paint(Graphics g, JComponent c)
      {
         Graphics2D g2 = (Graphics2D)g;

         AbstractButton b = (AbstractButton)c;
         ButtonModel bm = b.getModel();

         FontMetrics fm = g2.getFontMetrics();

         // define some colors for drawing
     
         final Color BLUE = new Color(47, 79, 136);
         final Color BLUE_LIGHT = new Color(107, 132, 181);
         final Color BLUE_DARK = new Color(32, 54, 94);
         final Color BORDER = new Color(152, 171, 206);

         Color backgroundColor;
         Color lineColor = BORDER; 

         //if (bm.isArmed() && !bm.isPressed())
         if (bm.isArmed())
         {
            backgroundColor = BLUE_LIGHT;
            b.setCursor(new Cursor(Cursor.HAND_CURSOR));
         }
         else if (bm.isPressed())
            backgroundColor = BLUE_LIGHT;
         else
            backgroundColor = BLUE;  // normal!

         // use a round rectangle for drawing

         Dimension d = c.getPreferredSize();
         int x = d.width - 1;    // x-coordinate of right edge
         int y = d.height - 1;   // y-coordinate of bottom edge
         final int LW = 2;       // line width
         int arc = y;

         g2.setColor(BLUE);
         g2.fillRect(0, 0, x + 1, y + 1);

         g2.setColor(lineColor);
         g2.fillRoundRect(0, 0, x, y, arc, arc);

         g2.setColor(backgroundColor);
         g2.fillRoundRect(LW, LW, x - 2 * LW, y - 2 * LW, arc, arc);

         // paint foreground

         g2.setColor(Color.white);
         Font f = fm.getFont();            
         g2.setFont(new Font(f.getName(), Font.BOLD, f.getSize()));
         String s = ((JButton)c).getText();
         int width = fm.stringWidth(s);
         int height = fm.getAscent();
         x = (d.width - width) / 2;
         y = (d.height - 2 + height - 2) / 2;
         g2.drawString(s, x, y);
      }

      /** Returns true if x,y is within the rounded rectangle representing
      * this button.
      */
      public boolean contains(JComponent c, int x, int y)
      {
         int w = c.getWidth();
         int h = c.getHeight();
         int arc = h;

         RoundRectangle2D.Double r =
            new RoundRectangle2D.Double(0, 0, w, h, arc, arc);

         return r.contains(x, y);
      }
   } 
}
